@@ -9,14 +9,14 @@
 # testing and support for generic AT commads, so use it at your own risk,
 # and without ANY warranty! Have fun.
 #
-# $Id: Modem.pm,v 1.36 2004/02/22 17:18:43 cosimo Exp $
+# $Id: Modem.pm,v 1.43 2004/11/10 09:20:04 cosimo Exp $
 
 package Device::Modem;
-$VERSION = sprintf '%d.%02d', q$Revision: 1.36 $ =~ /(\d)\.(\d+)/;
+$VERSION = sprintf '%d.%02d', q$Revision: 1.43 $ =~ /(\d)\.(\d+)/;
 
 BEGIN {
 
-    if( $^O =~ /Win/io ) {
+    if( index($^O, 'Win') >= 0 ) {   # MSWin32 (and not darwin, cygwin, ...)
 
         require Win32::SerialPort;
         import  Win32::SerialPort;
@@ -56,6 +56,8 @@ $Device::Modem::DATABITS = 8;
 $Device::Modem::STOPBITS = 1;
 $Device::Modem::PARITY   = 'none';
 $Device::Modem::TIMEOUT  = 500;     # milliseconds;
+$Device::Modem::WAITCYCLE= 50;
+$Device::Modem::READCHARS= 50;
 
 # Setup text and numerical response codes
 @Device::Modem::RESPONSE = ( 'OK', undef, 'RING', 'NO CARRIER', 'ERROR', undef, 'NO DIALTONE', 'BUSY' );
@@ -560,12 +562,12 @@ sub atsend {
 sub _answer {
     my $me = shift;
     my($expect, $timeout) = @_;
-    my $time_slice = 50;                       # single cycle wait time
+    my $time_slice = $Device::Modem::WAITCYCLE;     # single cycle wait time
 
     # If we expect something, we must first match against serial input
     my $done = (defined $expect and $expect ne '');
 
-#$me->log->write('debug', 'answer: expecting ['.($expect||'').']'.($timeout ? ' or '.($timeout/1000).' seconds timeout' : '' ) );
+$me->log->write('debug', 'answer: expecting ['.($expect||'').']'.($timeout ? ' or '.($timeout/1000).' seconds timeout' : '' ) );
 
     # Main read cycle
     my $cycles = 0;
@@ -575,17 +577,17 @@ sub _answer {
     my $end_time   = 0;
 
     # If timeout was defined, check max time (timeout is in milliseconds)
-    #$me->log->write('debug', 'answer: timeout value is '.$timeout);
+$me->log->write('debug', 'answer: timeout value is '.($timeout||'undef'));
 
     if( defined $timeout && $timeout > 0 ) {
         $end_time = $start_time + ($timeout / 1000);
 
-        #$me->log->write( debug => 'answer: end time set to '.$end_time );
+$me->log->write( debug => 'answer: end time set to '.$end_time );
     }
 
     do {
 
-        my($howmany, $what) = $me->port->read(10);
+        my($howmany, $what) = $me->port->read($Device::Modem::READCHARS);
 
         # Timeout count incremented only on empty readings
         if( defined $what && $howmany > 0 ) {
@@ -598,7 +600,6 @@ sub _answer {
                 $done = ( defined $answer && $answer =~ $expect ) ? 1 : 0;
             }
 
-            #$me->log->write('debug', 'time_slice='.$time_slice);
             $me->wait($time_slice) unless $done;
 
         # Check if we reached max time for timeout (only if end_time is defined)
@@ -612,12 +613,11 @@ sub _answer {
             $done = 1;
         }
 
-#$me->log->write('debug', 'done = '.$done );
-#$me->log->write('debug', 'end_time='.$end_time.' now='.time().' start_time='.$start_time );
+$me->log->write('debug', 'done='.$done.' end_time='.$end_time.' now='.time().' start_time='.$start_time );
 
       } while (not $done);
 
-    $me->log->write('debug', 'answer: read ['.($answer||'').']' );
+    $me->log->write('info', 'answer: read ['.($answer||'').']' );
 
     # Flush receive and trasmit buffers
     $me->port->purge_all;
@@ -651,20 +651,28 @@ sub parse_answer {
     my $buff = $me->answer( @_ );
 
     # Separate response code from information
-    my @buff = split /[\r\n]+/o, $buff;
+    if( defined $buff && $buff ne '' ) {
 
-    # Remove all empty lines before/after response
-    shift @buff while $buff[0]  =~ /^[\r\n]+/o;
-    pop   @buff while $buff[-1] =~ /^[\r\n]+/o;
+        my @buff = split /[\r\n]+/o, $buff;
 
-    # Extract responde code
-    my $code = pop @buff;
-    $buff = join( CR, @buff );
+        # Remove all empty lines before/after response
+        shift @buff while $buff[0]  =~ /^[\r\n]+/o;
+        pop   @buff while $buff[-1] =~ /^[\r\n]+/o;
 
-    return
-      wantarray
-      ? ($code, @buff)
-      : $buff;
+        # Extract responde code
+        $buff = join( CR, @buff );
+        my $code = pop @buff;
+
+        return
+            wantarray
+            ? ($code, @buff)
+            : $buff;
+            
+    } else {
+    
+        return '';
+
+    }
 
 }
 
@@ -808,6 +816,12 @@ that you can take as (yea) examples:
 
 Tests if modem is alive
 
+=item `examples/caller-id.pl'
+
+Waits for an incoming call and displays date, time and phone number of the caller.
+Normally this is available everywhere, but you should check your local phone line
+and settings.
+
 =item `examples/dial.pl'
 
 Dials a phone number and display result of call
@@ -816,6 +830,15 @@ Dials a phone number and display result of call
 
 (Very) poor man's minicom/hyperterminal utility
 
+=item `examples/xmodem.pl'
+
+First attempt at a test script to receive a file via xmodem protocol.
+Please be warned that this thing does not have a chance to work. It's
+only a (very low priority) work in progress...
+
+If you want to help out, be welcome!
+
+
 =back
 
 =head1 METHODS
